--- Used to get all available whisper commands.
-- @param sender The in game player who sent this command.
function sdkp:WCmdHelp(sender)
	self:SendWhisper(sender, "!bid [amount] - Bid amount.  If blank, amount = 1.  You can !bid full");
end

--- May be a whole number or the string "full".  Any text following the bid is ignored provided it's separated by a space.<br>
-- If the amount is less than 1, the amount is set to 1.  If it is greater than the full value for an item, it is set to full.
-- Amount may also be an empty string or nil, in which case an amount of 1 will be used.  This lets players simply "!bid".<br>
-- Players may not reduce the amount of their bid, however they may raise it. 
-- @param sender The in game player who sent this command.
-- @param amount The amount bid.
function sdkp:WCmdBid(sender, amount)
	local main = self:GetAltInfo(sender);
	local message = sender;
	if main ~= sender then 
		message = message.."("..main..")";
	end
	message = message.." bid "..amount;
	self:Print(message);
	
	if not sdkp.BIDDING then
		self:SendWhisper(sender, "Not currently accepting bids.");
		return;
	end
	
	if not amount or amount == ""  then
		amount = "1";
	else
		_, _, amount = string.find(amount, "^%s*([^%s]+)");
	end
	if amount:lower() == "full" then
		if self:IsShroudDkp() then
			amount = "50";
		else
			amount = self:GetDkpForItem(getglobal("sdkpeditbox"):GetText());
		end
	end
	
	local maxbid;
	if self:IsShroudDkp() then
		maxbid = 50;
	else
		maxbid = self:GetDkpForItem(getglobal("sdkpeditbox"):GetText());
	end
	amount = tonumber(amount);
	if amount and amount < 1 then 
		amount = 1; 
	end
	if amount == nil then 
		self:SendWhisper(sender, "Bid not understood.");
		return;
	elseif amount > maxbid then 
		amount = maxbid;
	end
	
	local current = self:GetCurrentBid(sender);
	if not current or current < amount then --effective then 
		self.db.profile.bids[sender:lower()] = amount; --effective;
		self:SendWhisper(sender, "Your bid of " .. amount .. " has been recorded.");
		self.BIDSCROLLFRAME:SortData();
	else
		self:SendWhisper(sender, "You have an existing bid of higher or equal value.");
	end
end

--- Notifies the mod of a players attendance during a boss kill check.  Claiming !here when no check in progress results in a sanction.
-- Each out of check !here will reset the sanction to it's full duration.
-- If a player is sanctioned, all claims of !here will be ignored.<br>
-- @param player The in game player who sent this command.
function sdkp:WCmdHere(player) 
	if self:IsAttCheck() then
		local who = player;
		
		local main = self:GetAltInfo(who);
		local mainRankString, mainRank = self:getPlayerRank(main);
		if not mainRank then
			sdkp:SendWhisper(who, "You are not in " .. GetGuildInfo("player") .. ".");
			return;
		elseif mainRank > 6 then
			sdkp:SendWhisper(who, main .. "'s rank is " .. mainRankString .. ". You can not earn DKP. If " .. who .. " is your alt, ask an officer to set your note to point to your main.");
			return;
		end
		
		local pdata = self:getPlayerData(player);
		if pdata then 
			player = pdata.cols[1].value;
		end
		
		if self.penaltybox[player] then 
			self:CancelTimer(self.penaltybox[player], true);
			self.penaltybox[player] = self:ScheduleTimer(function() 
				self.penaltybox[player] = nil;
				self:SendWhisper(who, "Your sanction has been lifted.");
			end, self:GetSanctionLen(), nil);
			return;
		end
		
		if self.checkingatt then
			local last_snap = sdkp:GetLastFullSnapshot();
			self:SendWhisper(who, player .. ", your presence is acknowledged.");
			self:Debug("Adding", player, "to", last_snap.name);
			self:CmdHandleManualSnapshot(player, "add", last_snap)
		else
			self:SendWhisper(who, "There is no attendance check in progress.  A whisper sanction of " .. string.format("%.1f", self:GetSanctionLen()/60) .. " minutes has been imposed upon you for automated whispering.  Timer will reset if you try again while sanctioned.");
			self:Debug("Sanctioning", player, "for automated whispers.", "len: ", self:GetSanctionLen(), "type(len):", type(self:GetSanctionLen()));
			self.penaltybox[player] = self:ScheduleTimer(function() 
				self.penaltybox[player] = nil;
				self:SendWhisper(who, "Your sanction has been lifted.");
			end, self:GetSanctionLen(), nil);
		end
	end
end

function sdkp:WCmdWaitlist(player, action)
	local waitlist = self.db.profile.waitlist
	local banlist = self.db.profile.banlist
	local main = self:GetAltInfo(player);
	local mainRankString, mainRank = self:getPlayerRank(main);
	
	action = string.lower(action);
	sdkp:Debug("WCmdWaitlist: " .. player .. " " .. action);
	
	if not action or action == "" or action == "help" then
		sdkp:SendWhisper(player, "Format is: !waitlist add||remove||status||show");	
		return;
	end
	if mainRank then
		if mainRank <= 6 then
			if action == "add" then
				if banlist[main] then
					sdkp:SendWhisper(player, main .. " has been banned for the night.");
				elseif waitlist[main] then
					sdkp:SendWhisper(player, main .. " is currently ON the waitlist, be ready.");
				else
					self:CmdHandleManualWaitlist(player, "add");
					sdkp:SendWhisper(player, main .. " has been added to the raid waitlist. You will be added to all boss kills while you are on the waitlist. If you are needed and can not be contacted, OR ARE STILL ON THE WAITLIST AT THE END OF THE NIGHT, you will lose all attendance and DKP for the night.");
				end
			elseif action == "remove" then
				if waitlist[main] then
					self:CmdHandleManualWaitlist(player, "remove");
					sdkp:SendWhisper(player, main .. " has been removed from the raid waitlist.");
				else
					sdkp:SendWhisper(player, main .. " was not on the waitlist, try to keep up. :P");
				end
			end
		end -- rank Trial or higher?
	end -- rank is nil?
		
	if action == "status" then
		if banlist[main] then
			sdkp:SendWhisper(player, main .. " has been banned for the night.");
		elseif waitlist[main] then
			sdkp:SendWhisper(player, main .. " is currently ON the waitlist, be ready.");
		else
			sdkp:SendWhisper(player, main .. " is NOT on the waitlist. Whisper !waitlist add to be added.");
		end
	elseif action == "show" or action == "who" then
		sdkp:SendWhisper(player, "Current waitlist: " .. self:CmdWaitlistString(waitlist));
	elseif (action == "add" or action == "remove") then
		if not mainRank then
			sdkp:SendWhisper(player, "You are not in " .. GetGuildInfo("player") .. ".");
		elseif mainRank > 6 then
			sdkp:SendWhisper(player, player .. "'s rank is " .. mainRankString .. ". You can not earn DKP.");
		end
	else
		sdkp:SendWhisper(player, "Not a valid waitlist command.");
		sdkp:SendWhisper(player, "Format is: !waitlist add||remove||status||show");
	end
	
end

--- Sends back a message to player announcing target's dkp.  If no target is specified, it uses player for target.
-- @param player The in game player who sent this command.
-- @param target The player to look up the dkp for.
function sdkp:WCmdDKP(player, target)
	if not target or target == "" then 
		target = player;
	end
	local pdata = self:getPlayerData(target);
	if pdata == nil then
		sdkp:SendWhisper(player, "Player '" .. target .. "' not on Dkp list.");
	else	
		local dkp = pdata.cols[7].value;
		if type(dkp) == "function" then 
			--data, cols, realrow, column, table
			dkp = dkp(unpack({self.BIDSCROLLFRAME.data, self.BIDSCROLLFRAME.cols, SDKPPLAYER[pdata.cols[1].value:lower()], 7, self.BIDSCROLLFRAME}));
		end
		sdkp:SendWhisper(player, pdata.cols[1].value .." Dkp: ".. dkp .."  TDA: ".. pdata.cols[3].value .."  Pres: ".. pdata.cols[5].value);
	end
end

--- Sends back a message to player announcing target's 30 day attendance.  If no target is specified, it uses player for target.
-- @param player The in game player who sent this command.
-- @param target The player to look up the dkp for.
function sdkp:WCmdAttn(player, target)
	if not target or target == "" then
		target = player;
	else
		local pdata = self:getPlayerData(target);
		if pdata == nil then
			sdkp:SendWhisper(player, "Player not on Attendance list.");
		else
			sdkp:SendWhisper(player, "30-day attendance percentage := ".. pdata.cols[3].value .. "%");
		end
	end
end
